@@ -20,14 +20,14 @@
 package main
 
 import (
+	"errors"
 	"fmt"
-	"sort"
-	"text/tabwriter"
-
-	"github.com/ubuntu-core/snappy/client"
-	"github.com/ubuntu-core/snappy/i18n"
+	"strings"
 
 	"github.com/jessevdk/go-flags"
+
+	"github.com/snapcore/snapd/client"
+	"github.com/snapcore/snapd/i18n"
 )
 
 var shortFindHelp = i18n.G("Finds packages to install")
@@ -35,10 +35,11 @@ var longFindHelp = i18n.G(`
 The find command queries the store for available packages.
 `)
 
-func getPrice(prices map[string]float64, currency string) string {
+func getPrice(prices map[string]float64, currency string) (float64, string, error) {
 	// If there are no prices, then the snap is free
 	if len(prices) == 0 {
-		return "-"
+		// TRANSLATORS: free as in gratis
+		return 0, "", errors.New(i18n.G("snap is free"))
 	}
 
 	// Look up the price by currency code
@@ -61,76 +62,80 @@ func getPrice(prices map[string]float64, currency string) string {
 		}
 	}
 
-	return fmt.Sprintf("%.2f%s", val, currency)
+	return val, currency, nil
+}
+
+type SectionName string
+
+func (s SectionName) Complete(match string) []flags.Completion {
+	cli := Client()
+	sections, err := cli.Sections()
+	if err != nil {
+		return nil
+	}
+	ret := make([]flags.Completion, 0, len(sections))
+	for _, s := range sections {
+		if strings.HasPrefix(s, match) {
+			ret = append(ret, flags.Completion{Item: s})
+		}
+	}
+	return ret
 }
 
 type cmdFind struct {
+	Private    bool        `long:"private"`
+	Section    SectionName `long:"section"`
 	Positional struct {
-		Query string `positional-arg-name:"<query>"`
+		Query string
 	} `positional-args:"yes"`
 }
 
 func init() {
 	addCommand("find", shortFindHelp, longFindHelp, func() flags.Commander {
 		return &cmdFind{}
+	}, map[string]string{
+		"private": i18n.G("Search private snaps"),
+		"section": i18n.G("Restrict the search to a given section"),
+	}, []argDesc{{name: i18n.G("<query>")}})
+}
+
+func (x *cmdFind) Execute(args []string) error {
+	if len(args) > 0 {
+		return ErrExtraArgs
+	}
+
+	// magic! `snap find` returns the featured snaps
+	if x.Positional.Query == "" && x.Section == "" {
+		x.Section = "featured"
+	}
+
+	return findSnaps(&client.FindOptions{
+		Private: x.Private,
+		Section: string(x.Section),
+		Query:   x.Positional.Query,
 	})
 }
 
-func hasPrices(snaps []*client.Snap) bool {
-	for _, snap := range snaps {
-		if len(snap.Prices) > 0 {
-			return true
-		}
-	}
-	return false
-}
-
-func printWithPrices(w *tabwriter.Writer, snaps []*client.Snap, suggestedCurrency string) {
-	fmt.Fprintln(w, i18n.G("Name\tVersion\tPrice\tSummary"))
-
-	for _, snap := range snaps {
-		price := getPrice(snap.Prices, suggestedCurrency)
-		fmt.Fprintf(w, "%s\t%s\t%s\t%s\n", snap.Name, snap.Version, price, snap.Summary)
-
-	}
-}
-
-func printNoPrices(w *tabwriter.Writer, snaps []*client.Snap) {
-	fmt.Fprintln(w, i18n.G("Name\tVersion\tSummary"))
-
-	for _, snap := range snaps {
-		fmt.Fprintf(w, "%s\t%s\t%s\n", snap.Name, snap.Version, snap.Summary)
-	}
-}
-
-func (x *cmdFind) Execute([]string) error {
+func findSnaps(opts *client.FindOptions) error {
 	cli := Client()
-	filter := client.SnapFilter{
-		Query:   x.Positional.Query,
-		Sources: []string{"store"},
-	}
-	snaps, resInfo, err := cli.FilterSnaps(filter)
+	snaps, resInfo, err := cli.Find(opts)
 	if err != nil {
 		return err
 	}
 
 	if len(snaps) == 0 {
-		if filter.Query == "" {
-			return fmt.Errorf("no snaps found")
-		}
-
-		return fmt.Errorf("no snaps found for %q", filter.Query)
+		// TRANSLATORS: the %q is the (quoted) query the user entered
+		return fmt.Errorf(i18n.G("no snaps found for %q"), opts.Query)
 	}
 
-	sort.Sort(snapsByName(snaps))
-
 	w := tabWriter()
 	defer w.Flush()
 
-	if hasPrices(snaps) {
-		printWithPrices(w, snaps, resInfo.SuggestedCurrency)
-	} else {
-		printNoPrices(w, snaps)
+	fmt.Fprintln(w, i18n.G("Name\tVersion\tDeveloper\tNotes\tSummary"))
+
+	for _, snap := range snaps {
+		// TODO: get snap.Publisher, so we can only show snap.Developer if it's different
+		fmt.Fprintf(w, "%s\t%s\t%s\t%s\t%s\n", snap.Name, snap.Version, snap.Developer, NotesFromRemote(snap, resInfo), snap.Summary)
 	}
 
 	return nil
