@@ -20,51 +20,43 @@
 package daemon
 
 import (
-	"io/ioutil"
-	"os"
-	"path/filepath"
-	"strconv"
-
 	. "gopkg.in/check.v1"
 
-	"github.com/ubuntu-core/snappy/dirs"
-	"github.com/ubuntu-core/snappy/interfaces"
-	"github.com/ubuntu-core/snappy/overlord/snapstate"
-	"github.com/ubuntu-core/snappy/snap"
+	"github.com/snapcore/snapd/interfaces"
+	"github.com/snapcore/snapd/overlord/snapstate"
+	"github.com/snapcore/snapd/snap"
+	"github.com/snapcore/snapd/snap/snaptest"
 )
 
 func (s *apiSuite) mockSnap(c *C, yamlText string) *snap.Info {
 	if s.d == nil {
 		panic("call s.daemon(c) in your test first")
 	}
+
+	snapInfo := snaptest.MockSnap(c, yamlText, "", &snap.SideInfo{Revision: snap.R(1)})
+	snap.AddImplicitSlots(snapInfo)
+
 	st := s.d.overlord.State()
 
 	st.Lock()
 	defer st.Unlock()
 
-	// Parse the yaml
-	snapInfo, err := snap.InfoFromSnapYaml([]byte(yamlText))
-	c.Assert(err, IsNil)
-	snap.AddImplicitSlots(snapInfo)
-
-	// Create on-disk yaml file (it is read by snapstate)
-	dname := filepath.Join(dirs.SnapSnapsDir, snapInfo.Name(),
-		strconv.Itoa(snapInfo.Revision), "meta")
-	fname := filepath.Join(dname, "snap.yaml")
-	err = os.MkdirAll(dname, 0755)
-	c.Assert(err, IsNil)
-	err = ioutil.WriteFile(fname, []byte(yamlText), 0644)
-	c.Assert(err, IsNil)
-
 	// Put a side info into the state
 	snapstate.Set(st, snapInfo.Name(), &snapstate.SnapState{
-		Active:   true,
-		Sequence: []*snap.SideInfo{{Revision: snapInfo.Revision}},
+		Active: true,
+		Sequence: []*snap.SideInfo{
+			{
+				RealName: snapInfo.Name(),
+				Revision: snapInfo.Revision,
+				SnapID:   "ididid",
+			},
+		},
+		Current: snapInfo.Revision,
 	})
 
 	// Put the snap into the interface repository
 	repo := s.d.overlord.InterfaceManager().Repository()
-	err = repo.AddSnap(snapInfo)
+	err := repo.AddSnap(snapInfo)
 	c.Assert(err, IsNil)
 	return snapInfo
 }
@@ -77,6 +69,11 @@ func (s *apiSuite) mockIface(c *C, iface interfaces.Interface) {
 	c.Assert(err, IsNil)
 }
 
+var simpleYaml = `
+name: simple
+version: 1
+`
+
 var consumerYaml = `
 name: consumer
 version: 1
@@ -112,3 +109,17 @@ slots:
   key: value
   label: label
 `
+
+var configYaml = `
+name: config-snap
+version: 1
+hooks:
+    configure:
+`
+var aliasYaml = `
+name: alias-snap
+version: 1
+apps:
+ app:
+  aliases: [alias1]
+`
