@@ -23,6 +23,7 @@ import (
 	"fmt"
 	"net"
 	"net/http"
+	"os/exec"
 	"strings"
 	"time"
 
@@ -30,26 +31,29 @@ import (
 	"github.com/gorilla/mux"
 	"gopkg.in/tomb.v2"
 
-	"github.com/ubuntu-core/snappy/logger"
-	"github.com/ubuntu-core/snappy/notifications"
-	"github.com/ubuntu-core/snappy/osutil"
-	"github.com/ubuntu-core/snappy/overlord"
-	"github.com/ubuntu-core/snappy/store"
+	"github.com/snapcore/snapd/dirs"
+	"github.com/snapcore/snapd/i18n/dumb"
+	"github.com/snapcore/snapd/logger"
+	"github.com/snapcore/snapd/osutil"
+	"github.com/snapcore/snapd/overlord"
+	"github.com/snapcore/snapd/overlord/auth"
+	"github.com/snapcore/snapd/overlord/state"
 )
 
 // A Daemon listens for requests and routes them to the right command
 type Daemon struct {
-	overlord *overlord.Overlord
-	listener net.Listener
-	tomb     tomb.Tomb
-	router   *mux.Router
-	hub      *notifications.Hub
+	Version       string
+	overlord      *overlord.Overlord
+	snapdListener net.Listener
+	snapListener  net.Listener
+	tomb          tomb.Tomb
+	router        *mux.Router
 	// enableInternalInterfaceActions controls if adding and removing slots and plugs is allowed.
 	enableInternalInterfaceActions bool
 }
 
 // A ResponseFunc handles one of the individual verbs for a method
-type ResponseFunc func(*Command, *http.Request) Response
+type ResponseFunc func(*Command, *http.Request, *auth.UserState) Response
 
 // A Command routes a request to an individual per-verb ResponseFUnc
 type Command struct {
@@ -59,46 +63,38 @@ type Command struct {
 	PUT    ResponseFunc
 	POST   ResponseFunc
 	DELETE ResponseFunc
-	// can sudoer do stuff?
-	SudoerOK bool
 	// can guest GET?
 	GuestOK bool
 	// can non-admin GET?
 	UserOK bool
-	//
+	// is this path accessible on the snapd-snap socket?
+	SnapOK bool
+
 	d *Daemon
 }
 
-var isUIDInAny = osutil.IsUIDInAny
-
-func (c *Command) canAccess(r *http.Request) bool {
-	state := c.d.overlord.State()
-	state.Lock()
-	_, err := UserFromRequest(state, r)
-	state.Unlock()
-	if err == nil {
-		// authenticated user does anything
+func (c *Command) canAccess(r *http.Request, user *auth.UserState) bool {
+	if user != nil {
+		// Authenticated users do anything for now.
 		return true
 	}
 
 	isUser := false
-	if uid, err := ucrednetGetUID(r.RemoteAddr); err == nil {
+	uid, err := ucrednetGetUID(r.RemoteAddr)
+	if err == nil {
 		if uid == 0 {
-			// superuser does anything
-			return true
-		}
-
-		if c.SudoerOK && isUIDInAny(uid, "sudo", "admin") {
-			// if user is in a group that grants sudo in
-			// the default install, and the command says
-			// that's ok, then it's ok
+			// Superuser does anything.
 			return true
 		}
 
 		isUser = true
+	} else if err != errNoUID {
+		logger.Noticef("unexpected error when attempting to get UID: %s", err)
+		return false
+	} else if c.SnapOK {
+		return true
 	}
 
-	// only superuser can modify
 	if r.Method != "GET" {
 		return false
 	}
@@ -115,8 +111,14 @@ func (c *Command) canAccess(r *http.Request) bool {
 }
 
 func (c *Command) ServeHTTP(w http.ResponseWriter, r *http.Request) {
-	if !c.canAccess(r) {
-		Forbidden("access denied").ServeHTTP(w, r)
+	state := c.d.overlord.State()
+	state.Lock()
+	// TODO Look at the error and fail if there's an attempt to authenticate with invalid data.
+	user, _ := UserFromRequest(state, r)
+	state.Unlock()
+
+	if !c.canAccess(r, user) {
+		Unauthorized("access denied").ServeHTTP(w, r)
 		return
 	}
 
@@ -135,7 +137,7 @@ func (c *Command) ServeHTTP(w http.ResponseWriter, r *http.Request) {
 	}
 
 	if rspf != nil {
-		rsp = rspf(c, r)
+		rsp = rspf(c, r, user)
 	}
 
 	rsp.ServeHTTP(w, r)
@@ -181,11 +183,24 @@ func (d *Daemon) Init() error {
 		return err
 	}
 
-	if len(listeners) != 1 {
-		return fmt.Errorf("daemon does not handle %d listeners right now, just one", len(listeners))
+	listenerMap := make(map[string]net.Listener)
+
+	for _, listener := range listeners {
+		listenerMap[listener.Addr().String()] = listener
 	}
 
-	d.listener = &ucrednetListener{listeners[0]}
+	// The SnapdSocket is required-- without it, die.
+	if listener, ok := listenerMap[dirs.SnapdSocket]; ok {
+		d.snapdListener = &ucrednetListener{listener}
+	} else {
+		return fmt.Errorf("daemon is missing the listener for %s", dirs.SnapdSocket)
+	}
+
+	// Note that the SnapSocket listener does not use ucrednet. We use the lack
+	// of remote information as an indication that the request originated with
+	// this socket. This listener may also be nil if that socket wasn't among
+	// the listeners, so check it before using it.
+	d.snapListener = listenerMap[dirs.SnapSocket]
 
 	d.addRoutes()
 
@@ -199,7 +214,6 @@ func (d *Daemon) addRoutes() {
 
 	for _, c := range api {
 		c.d = d
-		logger.Debugf("adding %s", c.Path)
 		d.router.Handle(c.Path, c).Name(c.Path)
 	}
 
@@ -208,12 +222,41 @@ func (d *Daemon) addRoutes() {
 	d.router.NotFoundHandler = NotFound("not found")
 }
 
+var shutdownMsg = i18n.G("reboot scheduled to update the system - temporarily cancel with 'sudo shutdown -c'")
+
 // Start the Daemon
 func (d *Daemon) Start() {
+	// die when asked to restart (systemd should get us back up!)
+	d.overlord.SetRestartHandler(func(t state.RestartType) {
+		switch t {
+		case state.RestartDaemon:
+			d.tomb.Kill(nil)
+		case state.RestartSystem:
+			cmd := exec.Command("shutdown", "+10", "-r", shutdownMsg)
+			if out, err := cmd.CombinedOutput(); err != nil {
+				logger.Noticef("%s", osutil.OutputErr(out, err))
+			}
+		default:
+			logger.Noticef("internal error: restart handler called with unknown restart type: %v", t)
+			d.tomb.Kill(nil)
+		}
+	})
+
 	// the loop runs in its own goroutine
 	d.overlord.Loop()
+
 	d.tomb.Go(func() error {
-		if err := http.Serve(d.listener, logit(d.router)); err != nil && d.tomb.Err() == tomb.ErrStillAlive {
+		if d.snapListener != nil {
+			d.tomb.Go(func() error {
+				if err := http.Serve(d.snapListener, logit(d.router)); err != nil && d.tomb.Err() == tomb.ErrStillAlive {
+					return err
+				}
+
+				return nil
+			})
+		}
+
+		if err := http.Serve(d.snapdListener, logit(d.router)); err != nil && d.tomb.Err() == tomb.ErrStillAlive {
 			return err
 		}
 
@@ -224,8 +267,12 @@ func (d *Daemon) Start() {
 // Stop shuts down the Daemon
 func (d *Daemon) Stop() error {
 	d.tomb.Kill(nil)
-	d.listener.Close()
+	d.snapdListener.Close()
+	if d.snapListener != nil {
+		d.snapListener.Close()
+	}
 	d.overlord.Stop()
+
 	return d.tomb.Wait()
 }
 
@@ -234,18 +281,6 @@ func (d *Daemon) Dying() <-chan struct{} {
 	return d.tomb.Dying()
 }
 
-func (d *Daemon) auther(r *http.Request) (store.Authenticator, error) {
-	overlord := d.overlord
-	state := overlord.State()
-	state.Lock()
-	user, err := UserFromRequest(state, r)
-	state.Unlock()
-	if err != nil {
-		return nil, err
-	}
-	return user.Authenticator(), nil
-}
-
 // New Daemon
 func New() (*Daemon, error) {
 	ovld, err := overlord.New()
@@ -254,7 +289,6 @@ func New() (*Daemon, error) {
 	}
 	return &Daemon{
 		overlord: ovld,
-		hub:      notifications.NewHub(),
 		// TODO: Decide when this should be disabled by default.
 		enableInternalInterfaceActions: true,
 	}, nil
