@@ -20,7 +20,7 @@
 package builtin
 
 import (
-	"github.com/ubuntu-core/snappy/interfaces"
+	"github.com/snapcore/snapd/interfaces"
 )
 
 // http://bazaar.launchpad.net/~ubuntu-security/ubuntu-core-security/trunk/view/head:/data/apparmor/policygroups/ubuntu-core/16.04/home
@@ -34,14 +34,19 @@ const homeConnectedPlugAppArmor = `
 # Allow read access to toplevel $HOME for the user
 owner @{HOME}/ r,
 
-# Allow read/write access to all non-hidden files that aren't in ~/snaps/
+# Allow read/write access to all files in @{HOME}, except snap application
+# data in @{HOME}/snaps and toplevel hidden directories in @{HOME}.
 owner @{HOME}/[^s.]**             rwk,
 owner @{HOME}/s[^n]**             rwk,
 owner @{HOME}/sn[^a]**            rwk,
 owner @{HOME}/sna[^p]**           rwk,
-owner @{HOME}/snap[^s]**          rwk,
-# allow creating a few files not caught above
-owner @{HOME}/{s,sn,sna,snap}{,/} rwk,
+# Allow creating a few files not caught above
+owner @{HOME}/{s,sn,sna}{,/} rwk,
+
+# Allow access to gvfs mounts for files owned by the user (including hidden
+# files; only allow writes to files, not the mount point).
+owner /run/user/[0-9]*/gvfs/**   r,
+owner /run/user/[0-9]*/gvfs/*/** w,
 `
 
 // NewHomeInterface returns a new "home" interface.
