@@ -26,7 +26,7 @@ import (
 	"strings"
 	"sync"
 
-	"github.com/ubuntu-core/snappy/snap"
+	"github.com/snapcore/snapd/snap"
 )
 
 // Repository stores all known snappy plugs and slots and ifaces.
@@ -201,7 +201,7 @@ func (r *Repository) Slots(snapName string) []*Slot {
 	return result
 }
 
-// Slot returns the specified plug slot from the named snap.
+// Slot returns the specified slot from the named snap.
 func (r *Repository) Slot(snapName, slotName string) *Slot {
 	r.m.Lock()
 	defer r.m.Unlock()
@@ -252,7 +252,7 @@ func (r *Repository) RemoveSlot(snapName, slotName string) error {
 	// Ensure that such slot exists
 	slot := r.slots[snapName][slotName]
 	if slot == nil {
-		return fmt.Errorf("cannot remove plug slot %q from snap %q, no such slot", slotName, snapName)
+		return fmt.Errorf("cannot remove slot %q from snap %q, no such slot", slotName, snapName)
 	}
 	// Ensure that the slot is not using any plugs
 	if len(r.slotPlugs[slot]) > 0 {
@@ -265,12 +265,84 @@ func (r *Repository) RemoveSlot(snapName, slotName string) error {
 	return nil
 }
 
-// Connect establishes a connection between a plug and a slot.
-// The plug and the slot must have the same interface.
-func (r *Repository) Connect(plugSnapName, plugName, slotSnapName, slotName string) error {
+// ResolveConnect resolves potentially missing plug or slot names and returns a
+// fully populated connection reference.
+func (r *Repository) ResolveConnect(plugSnapName, plugName, slotSnapName, slotName string) (ConnRef, error) {
 	r.m.Lock()
 	defer r.m.Unlock()
 
+	ref := ConnRef{}
+
+	if plugSnapName == "" {
+		return ref, fmt.Errorf("cannot resolve connection, plug snap name is empty")
+	}
+	if plugName == "" {
+		return ref, fmt.Errorf("cannot resolve connection, plug name is empty")
+	}
+	// Ensure that such plug exists
+	plug := r.plugs[plugSnapName][plugName]
+	if plug == nil {
+		return ref, fmt.Errorf("snap %q has no plug named %q", plugSnapName, plugName)
+	}
+
+	if slotSnapName == "" {
+		// Use the core snap if the slot-side snap name is empty
+		switch {
+		case r.slots["core"] != nil:
+			slotSnapName = "core"
+		case r.slots["ubuntu-core"] != nil:
+			slotSnapName = "ubuntu-core"
+		default:
+			// XXX: perhaps this should not be an error and instead it should
+			// silently assume "core" now?
+			return ref, fmt.Errorf("cannot resolve connection, slot snap name is empty")
+		}
+	}
+	if slotName == "" {
+		// Find the unambiguous slot that satisfies plug requirements
+		var candidates []string
+		for candidateSlotName, candidateSlot := range r.slots[slotSnapName] {
+			// TODO: use some smarter matching (e.g. against $attrs)
+			if candidateSlot.Interface == plug.Interface {
+				candidates = append(candidates, candidateSlotName)
+			}
+		}
+		switch len(candidates) {
+		case 0:
+			return ref, fmt.Errorf("snap %q has no %q interface slots", slotSnapName, plug.Interface)
+		case 1:
+			slotName = candidates[0]
+		default:
+			sort.Strings(candidates)
+			return ref, fmt.Errorf("snap %q has multiple %q interface slots: %s", slotSnapName, plug.Interface, strings.Join(candidates, ", "))
+		}
+	}
+
+	// Ensure that such slot exists
+	slot := r.slots[slotSnapName][slotName]
+	if slot == nil {
+		return ref, fmt.Errorf("snap %q has no slot named %q", slotSnapName, slotName)
+	}
+	// Ensure that plug and slot are compatible
+	if slot.Interface != plug.Interface {
+		return ref, fmt.Errorf("cannot connect %s:%s (%q interface) to %s:%s (%q interface)",
+			plugSnapName, plugName, plug.Interface, slotSnapName, slotName, slot.Interface)
+	}
+	ref = ConnRef{PlugRef: plug.Ref(), SlotRef: slot.Ref()}
+	return ref, nil
+}
+
+// Connect establishes a connection between a plug and a slot.
+// The plug and the slot must have the same interface.
+func (r *Repository) Connect(ref ConnRef) error {
+	r.m.Lock()
+	defer r.m.Unlock()
+
+	plugSnapName := ref.PlugRef.Snap
+	plugName := ref.PlugRef.Name
+	slotSnapName := ref.SlotRef.Snap
+	slotName := ref.SlotRef.Name
+
 	// Ensure that such plug exists
 	plug := r.plugs[plugSnapName][plugName]
 	if plug == nil {
@@ -307,85 +379,101 @@ func (r *Repository) Connect(plugSnapName, plugName, slotSnapName, slotName stri
 
 // Disconnect disconnects the named plug from the slot of the given snap.
 //
-// Disconnect has three modes of operation that depend on the passed arguments:
-//
-// - If all the arguments are specified then Disconnect() finds a specific slot
-//   and a specific plug and disconnects that plug from that plug slot. It is
-//   an error if plug or plug slot cannot be found or if the connect does not
-//   exist.
-// - If plugSnapName and plugName are empty then Disconnect() finds the specified
-//   slot and disconnects all the plugs connected there. It is not an error if
-//   there are no such plugs but it is still an error if the plug slot does
-//   not exist.
-// - If plugSnapName, plugName and slotName are all empty then Disconnect finds
-//   the specified snap (designated by slotSnapName) and disconnects all the plugs
-//   from all the slots found therein. It is not an error if there are no
-//   such plugs but it is still an error if the snap does not exist or has no
-//   slots at all.
+// Disconnect() finds a specific slot and a specific plug and disconnects that
+// plug from that slot. It is an error if plug or slot cannot be found or if
+// the connect does not exist.
 func (r *Repository) Disconnect(plugSnapName, plugName, slotSnapName, slotName string) error {
 	r.m.Lock()
 	defer r.m.Unlock()
 
-	switch {
-	case plugSnapName == "" && plugName == "" && slotName == "":
-		// Disconnect everything from slotSnapName
-		return r.disconnectEverythingFromSnap(slotSnapName)
-	case plugSnapName == "" && plugName == "":
-		// Disconnect everything from slotSnapName:slotName
-		return r.disconnectEverythingFromSlot(slotSnapName, slotName)
-	default:
-		return r.disconnectPlugFromSlot(plugSnapName, plugName, slotSnapName, slotName)
+	// Sanity check
+	if plugSnapName == "" {
+		return fmt.Errorf("cannot disconnect, plug snap name is empty")
+	}
+	if plugName == "" {
+		return fmt.Errorf("cannot disconnect, plug name is empty")
+	}
+	if slotSnapName == "" {
+		return fmt.Errorf("cannot disconnect, slot snap name is empty")
+	}
+	if slotName == "" {
+		return fmt.Errorf("cannot disconnect, slot name is empty")
 	}
 
-}
-
-// disconnectEverythingFromSnap finds a specific snap and disconnects all the plugs connected to all the slots therein.
-func (r *Repository) disconnectEverythingFromSnap(slotSnapName string) error {
-	if _, ok := r.slots[slotSnapName]; !ok {
-		return fmt.Errorf("cannot disconnect plug from snap %q, no such snap", slotSnapName)
-	}
-	for _, slot := range r.slots[slotSnapName] {
-		for plug := range r.slotPlugs[slot] {
-			r.disconnect(plug, slot)
-		}
-	}
-	return nil
-}
-
-// disconnectEverythingFromSlot finds a specific plug slot and disconnects all the plugs connected there.
-func (r *Repository) disconnectEverythingFromSlot(slotSnapName, slotName string) error {
-	// Ensure that such slot exists
-	slot := r.slots[slotSnapName][slotName]
-	if slot == nil {
-		return fmt.Errorf("cannot disconnect plug from slot %q from snap %q, no such slot", slotName, slotSnapName)
-	}
-	for plug := range r.slotPlugs[slot] {
-		r.disconnect(plug, slot)
-	}
-	return nil
-}
-
-// disconnectPlugFromSlot finds a specific plug slot and plug and disconnects it.
-func (r *Repository) disconnectPlugFromSlot(plugSnapName, plugName, slotSnapName, slotName string) error {
 	// Ensure that such plug exists
 	plug := r.plugs[plugSnapName][plugName]
 	if plug == nil {
-		return fmt.Errorf("cannot disconnect plug %q from snap %q, no such plug", plugName, plugSnapName)
+		return fmt.Errorf("snap %q has no plug named %q", plugSnapName, plugName)
 	}
 	// Ensure that such slot exists
 	slot := r.slots[slotSnapName][slotName]
 	if slot == nil {
-		return fmt.Errorf("cannot disconnect plug from slot %q from snap %q, no such slot", slotName, slotSnapName)
+		return fmt.Errorf("snap %q has no slot named %q", slotSnapName, slotName)
 	}
 	// Ensure that slot and plug are connected
 	if !r.slotPlugs[slot][plug] {
-		return fmt.Errorf("cannot disconnect plug %q from snap %q from slot %q from snap %q, it is not connected",
-			plugName, plugSnapName, slotName, slotSnapName)
+		return fmt.Errorf("cannot disconnect %s:%s from %s:%s, it is not connected",
+			plugSnapName, plugName, slotSnapName, slotName)
 	}
 	r.disconnect(plug, slot)
 	return nil
 }
 
+// Connected returns references for all connections that are currently
+// established with the provided plug or slot.
+func (r *Repository) Connected(snapName, plugOrSlotName string) ([]ConnRef, error) {
+	r.m.Lock()
+	defer r.m.Unlock()
+
+	if snapName == "" {
+		// Look up the core snap if no snap name was given
+		switch {
+		case r.slots["core"] != nil:
+			snapName = "core"
+		case r.slots["ubuntu-core"] != nil:
+			snapName = "ubuntu-core"
+		default:
+			return nil, fmt.Errorf("snap name is empty")
+		}
+	}
+	var conns []ConnRef
+	if plugOrSlotName == "" {
+		return nil, fmt.Errorf("plug or slot name is empty")
+	}
+	// Check if plugOrSlotName actually maps to anything
+	if r.plugs[snapName][plugOrSlotName] == nil && r.slots[snapName][plugOrSlotName] == nil {
+		return nil, fmt.Errorf("snap %q has no plug or slot named %q", snapName, plugOrSlotName)
+	}
+	// Collect all the relevant connections
+	if plug, ok := r.plugs[snapName][plugOrSlotName]; ok {
+		for _, slotRef := range plug.Connections {
+			connRef := ConnRef{PlugRef: plug.Ref(), SlotRef: slotRef}
+			conns = append(conns, connRef)
+		}
+	}
+	if slot, ok := r.slots[snapName][plugOrSlotName]; ok {
+		for _, plugRef := range slot.Connections {
+			connRef := ConnRef{PlugRef: plugRef, SlotRef: slot.Ref()}
+			conns = append(conns, connRef)
+		}
+	}
+	return conns, nil
+}
+
+// DisconnectAll disconnects all provided connection references.
+func (r *Repository) DisconnectAll(conns []ConnRef) {
+	r.m.Lock()
+	defer r.m.Unlock()
+
+	for _, conn := range conns {
+		plug := r.plugs[conn.PlugRef.Snap][conn.PlugRef.Name]
+		slot := r.slots[conn.SlotRef.Snap][conn.SlotRef.Name]
+		if plug != nil && slot != nil {
+			r.disconnect(plug, slot)
+		}
+	}
+}
+
 // disconnect disconnects a plug from a slot.
 func (r *Repository) disconnect(plug *Plug, slot *Slot) {
 	delete(r.slotPlugs[slot], plug)
@@ -451,8 +539,8 @@ func (r *Repository) Interfaces() *Interfaces {
 }
 
 // SecuritySnippetsForSnap collects all of the snippets of a given security
-// system that affect a given snap. The return value is indexed by app name
-// within that snap.
+// system that affect a given snap. The return value is indexed by app/hook
+// security tag within that snap.
 func (r *Repository) SecuritySnippetsForSnap(snapName string, securitySystem SecuritySystem) (map[string][][]byte, error) {
 	r.m.Lock()
 	defer r.m.Unlock()
@@ -460,6 +548,24 @@ func (r *Repository) SecuritySnippetsForSnap(snapName string, securitySystem Sec
 	return r.securitySnippetsForSnap(snapName, securitySystem)
 }
 
+func addSnippet(snapName, uniqueName string, apps map[string]*snap.AppInfo, hooks map[string]*snap.HookInfo, snippets map[string][][]byte, snippet []byte) {
+	if len(snippet) == 0 {
+		return
+	}
+	for appName := range apps {
+		securityTag := snap.AppSecurityTag(snapName, appName)
+		snippets[securityTag] = append(snippets[securityTag], snippet)
+	}
+	for hookName := range hooks {
+		securityTag := snap.HookSecurityTag(snapName, hookName)
+		snippets[securityTag] = append(snippets[securityTag], snippet)
+	}
+	if len(apps) == 0 && len(hooks) == 0 {
+		securityTag := snap.NoneSecurityTag(snapName, uniqueName)
+		snippets[securityTag] = append(snippets[securityTag], snippet)
+	}
+}
+
 func (r *Repository) securitySnippetsForSnap(snapName string, securitySystem SecuritySystem) (map[string][][]byte, error) {
 	var snippets = make(map[string][][]byte)
 	// Find all of the slots that affect this snap because of plug connection.
@@ -470,23 +576,15 @@ func (r *Repository) securitySnippetsForSnap(snapName string, securitySystem Sec
 		if err != nil {
 			return nil, err
 		}
-		if snippet != nil {
-			for appName := range slot.Apps {
-				snippets[appName] = append(snippets[appName], snippet)
-			}
-		}
+		addSnippet(snapName, slot.Name, slot.Apps, nil, snippets, snippet)
+
 		// Add connection-specific snippet specific to each plug
 		for plug := range r.slotPlugs[slot] {
 			snippet, err := iface.ConnectedSlotSnippet(plug, slot, securitySystem)
 			if err != nil {
 				return nil, err
 			}
-			if snippet == nil {
-				continue
-			}
-			for appName := range slot.Apps {
-				snippets[appName] = append(snippets[appName], snippet)
-			}
+			addSnippet(snapName, slot.Name, slot.Apps, nil, snippets, snippet)
 		}
 	}
 	// Find all of the plugs that affect this snap because of slot connection
@@ -497,23 +595,15 @@ func (r *Repository) securitySnippetsForSnap(snapName string, securitySystem Sec
 		if err != nil {
 			return nil, err
 		}
-		if snippet != nil {
-			for appName := range plug.Apps {
-				snippets[appName] = append(snippets[appName], snippet)
-			}
-		}
+		addSnippet(snapName, plug.Name, plug.Apps, plug.Hooks, snippets, snippet)
+
 		// Add connection-specific snippet specific to each slot
 		for slot := range r.plugSlots[plug] {
 			snippet, err := iface.ConnectedPlugSnippet(plug, slot, securitySystem)
 			if err != nil {
 				return nil, err
 			}
-			if snippet == nil {
-				continue
-			}
-			for appName := range plug.Apps {
-				snippets[appName] = append(snippets[appName], snippet)
-			}
+			addSnippet(snapName, plug.Name, plug.Apps, plug.Hooks, snippets, snippet)
 		}
 	}
 	return snippets, nil
@@ -655,36 +745,10 @@ func (r *Repository) RemoveSnap(snapName string) error {
 	return nil
 }
 
-// AutoConnectBlacklist returns plug names that should not be auto-connected.
-//
-// Plug is blacklisted if it has no connections despite using an auto-connected
-// interface. That implies it was manually disconnected.
-func (r *Repository) AutoConnectBlacklist(snapName string) map[string]bool {
-	r.m.Lock()
-	defer r.m.Unlock()
-
-	var blacklist map[string]bool
-
-	for plugName, plug := range r.plugs[snapName] {
-		iface := r.ifaces[plug.Interface]
-		if !iface.AutoConnect() {
-			continue
-		}
-		if len(r.plugSlots[plug]) != 0 {
-			continue
-		}
-		if blacklist == nil {
-			blacklist = make(map[string]bool)
-		}
-		blacklist[plugName] = true
-	}
-	return blacklist
-}
-
 // DisconnectSnap disconnects all the connections to and from a given snap.
 //
-// The return value is a list of snap.Info's that were affected.
-func (r *Repository) DisconnectSnap(snapName string) ([]*snap.Info, error) {
+// The return value is a list of names that were affected.
+func (r *Repository) DisconnectSnap(snapName string) ([]string, error) {
 	r.m.Lock()
 	defer r.m.Unlock()
 
@@ -706,16 +770,17 @@ func (r *Repository) DisconnectSnap(snapName string) ([]*snap.Info, error) {
 		}
 	}
 
-	result := make([]*snap.Info, 0, len(seen))
+	result := make([]string, 0, len(seen))
 	for info := range seen {
-		result = append(result, info)
+		result = append(result, info.Name())
 	}
+	sort.Strings(result)
 	return result, nil
 }
 
 // AutoConnectCandidates finds and returns viable auto-connection candidates
 // for a given plug.
-func (r *Repository) AutoConnectCandidates(plugSnapName, plugName string) []*Slot {
+func (r *Repository) AutoConnectCandidates(plugSnapName, plugName string, policyCheck func(*Plug, *Slot) bool) []*Slot {
 	r.m.Lock()
 	defer r.m.Unlock()
 
@@ -723,13 +788,20 @@ func (r *Repository) AutoConnectCandidates(plugSnapName, plugName string) []*Slo
 	if plug == nil {
 		return nil
 	}
-	if r.ifaces[plug.Interface].AutoConnect() == false {
-		return nil
-	}
+
 	var candidates []*Slot
 	for _, slotsForSnap := range r.slots {
 		for _, slot := range slotsForSnap {
-			if slot.Snap.Type == snap.TypeOS && slot.Interface == plug.Interface {
+			if slot.Interface != plug.Interface {
+				continue
+			}
+
+			// declaration based checks disallow
+			if !policyCheck(plug, slot) {
+				continue
+			}
+
+			if r.ifaces[plug.Interface].AutoConnect(plug, slot) {
 				candidates = append(candidates, slot)
 			}
 		}
