@@ -26,90 +26,67 @@ import (
 
 	. "gopkg.in/check.v1"
 
-	"github.com/ubuntu-core/snappy/dirs"
-	"github.com/ubuntu-core/snappy/interfaces"
-	"github.com/ubuntu-core/snappy/interfaces/seccomp"
-	"github.com/ubuntu-core/snappy/snap"
-	"github.com/ubuntu-core/snappy/testutil"
+	"github.com/snapcore/snapd/dirs"
+	"github.com/snapcore/snapd/interfaces"
+	"github.com/snapcore/snapd/interfaces/backendtest"
+	"github.com/snapcore/snapd/interfaces/seccomp"
+	"github.com/snapcore/snapd/snap/snaptest"
+	"github.com/snapcore/snapd/testutil"
 )
 
 type backendSuite struct {
-	backend *seccomp.Backend
-	repo    *interfaces.Repository
-	iface   *interfaces.TestInterface
-	rootDir string
+	backendtest.BackendSuite
 }
 
-var _ = Suite(&backendSuite{
-	backend: &seccomp.Backend{},
-})
+var _ = Suite(&backendSuite{})
+
+var testedConfinementOpts = []interfaces.ConfinementOptions{
+	{},
+	{DevMode: true},
+	{JailMode: true},
+	{Classic: true},
+}
 
 func (s *backendSuite) SetUpTest(c *C) {
-	// Isolate this test to a temporary directory
-	s.rootDir = c.MkDir()
-	dirs.SetRootDir(s.rootDir)
+	s.Backend = &seccomp.Backend{}
+	s.BackendSuite.SetUpTest(c)
+
 	// Prepare a directory for seccomp profiles.
 	// NOTE: Normally this is a part of the OS snap.
 	err := os.MkdirAll(dirs.SnapSeccompDir, 0700)
 	c.Assert(err, IsNil)
-	// Create a fresh repository for each test
-	s.repo = interfaces.NewRepository()
-	s.iface = &interfaces.TestInterface{InterfaceName: "iface"}
-	err = s.repo.AddInterface(s.iface)
-	c.Assert(err, IsNil)
 }
 
 func (s *backendSuite) TearDownTest(c *C) {
-	dirs.SetRootDir("/")
+	s.BackendSuite.TearDownTest(c)
 }
 
 // Tests for Setup() and Remove()
-const sambaYamlV1 = `
-name: samba
-version: 1
-developer: acme
-apps:
-    smbd:
-slots:
-    iface:
-`
-const sambaYamlV1WithNmbd = `
-name: samba
-version: 1
-developer: acme
-apps:
-    smbd:
-    nmbd:
-slots:
-    iface:
-`
-const sambaYamlV2 = `
-name: samba
-version: 2
-developer: acme
-apps:
-    smbd:
-slots:
-    iface:
-`
-
 func (s *backendSuite) TestName(c *C) {
-	c.Check(s.backend.Name(), Equals, "seccomp")
+	c.Check(s.Backend.Name(), Equals, "seccomp")
 }
 
 func (s *backendSuite) TestInstallingSnapWritesProfiles(c *C) {
-	devMode := false
-	s.installSnap(c, devMode, sambaYamlV1)
+	s.InstallSnap(c, interfaces.ConfinementOptions{}, backendtest.SambaYamlV1, 0)
 	profile := filepath.Join(dirs.SnapSeccompDir, "snap.samba.smbd")
 	// file called "snap.sambda.smbd" was created
 	_, err := os.Stat(profile)
 	c.Check(err, IsNil)
 }
 
+func (s *backendSuite) TestInstallingSnapWritesHookProfiles(c *C) {
+	s.InstallSnap(c, interfaces.ConfinementOptions{}, backendtest.HookYaml, 0)
+	profile := filepath.Join(dirs.SnapSeccompDir, "snap.foo.hook.configure")
+
+	// Verify that profile named "snap.foo.hook.configure" was created.
+	_, err := os.Stat(profile)
+	c.Check(err, IsNil)
+}
+
 func (s *backendSuite) TestRemovingSnapRemovesProfiles(c *C) {
-	for _, devMode := range []bool{true, false} {
-		snapInfo := s.installSnap(c, devMode, sambaYamlV1)
-		s.removeSnap(c, snapInfo)
+	for _, opts := range testedConfinementOpts {
+		snapInfo := s.InstallSnap(c, opts, backendtest.SambaYamlV1, 0)
+		s.RemoveSnap(c, snapInfo)
 		profile := filepath.Join(dirs.SnapSeccompDir, "snap.samba.smbd")
 		// file called "snap.sambda.smbd" was removed
 		_, err := os.Stat(profile)
@@ -117,35 +94,72 @@ func (s *backendSuite) TestRemovingSnapRemovesProfiles(c *C) {
 	}
 }
 
-func (s *backendSuite) TestUpdatingSnapToOneWithMoreApps(c *C) {
-	for _, devMode := range []bool{true, false} {
-		snapInfo := s.installSnap(c, devMode, sambaYamlV1)
-		snapInfo = s.updateSnap(c, snapInfo, devMode, sambaYamlV1WithNmbd)
-		profile := filepath.Join(dirs.SnapSeccompDir, "snap.samba.nmbd")
-		// file called "snap.sambda.nmbd" was created
+func (s *backendSuite) TestRemovingSnapRemovesHookProfiles(c *C) {
+	for _, opts := range testedConfinementOpts {
+		snapInfo := s.InstallSnap(c, opts, backendtest.HookYaml, 0)
+		s.RemoveSnap(c, snapInfo)
+		profile := filepath.Join(dirs.SnapSeccompDir, "snap.foo.hook.configure")
+
+		// Verify that profile "snap.foo.hook.configure" was removed.
 		_, err := os.Stat(profile)
+		c.Check(os.IsNotExist(err), Equals, true)
+	}
+}
+
+func (s *backendSuite) TestUpdatingSnapToOneWithMoreApps(c *C) {
+	for _, opts := range testedConfinementOpts {
+		snapInfo := s.InstallSnap(c, opts, backendtest.SambaYamlV1, 0)
+		snapInfo = s.UpdateSnap(c, snapInfo, opts, backendtest.SambaYamlV1WithNmbd, 0)
+		profile := filepath.Join(dirs.SnapSeccompDir, "snap.samba.nmbd")
+		_, err := os.Stat(profile)
+		// file called "snap.sambda.nmbd" was created
 		c.Check(err, IsNil)
-		s.removeSnap(c, snapInfo)
+		s.RemoveSnap(c, snapInfo)
+	}
+}
+
+func (s *backendSuite) TestUpdatingSnapToOneWithHooks(c *C) {
+	for _, opts := range testedConfinementOpts {
+		snapInfo := s.InstallSnap(c, opts, backendtest.SambaYamlV1, 0)
+		snapInfo = s.UpdateSnap(c, snapInfo, opts, backendtest.SambaYamlWithHook, 0)
+		profile := filepath.Join(dirs.SnapSeccompDir, "snap.samba.hook.configure")
+
+		_, err := os.Stat(profile)
+		// Verify that profile "snap.samba.hook.configure" was created.
+		c.Check(err, IsNil)
+		s.RemoveSnap(c, snapInfo)
 	}
 }
 
 func (s *backendSuite) TestUpdatingSnapToOneWithFewerApps(c *C) {
-	for _, devMode := range []bool{true, false} {
-		snapInfo := s.installSnap(c, devMode, sambaYamlV1WithNmbd)
-		snapInfo = s.updateSnap(c, snapInfo, devMode, sambaYamlV1)
+	for _, opts := range testedConfinementOpts {
+		snapInfo := s.InstallSnap(c, opts, backendtest.SambaYamlV1WithNmbd, 0)
+		snapInfo = s.UpdateSnap(c, snapInfo, opts, backendtest.SambaYamlV1, 0)
 		profile := filepath.Join(dirs.SnapSeccompDir, "snap.samba.nmbd")
 		// file called "snap.sambda.nmbd" was removed
 		_, err := os.Stat(profile)
 		c.Check(os.IsNotExist(err), Equals, true)
-		s.removeSnap(c, snapInfo)
+		s.RemoveSnap(c, snapInfo)
+	}
+}
+
+func (s *backendSuite) TestUpdatingSnapToOneWithNoHooks(c *C) {
+	for _, opts := range testedConfinementOpts {
+		snapInfo := s.InstallSnap(c, opts, backendtest.SambaYamlWithHook, 0)
+		snapInfo = s.UpdateSnap(c, snapInfo, opts, backendtest.SambaYamlV1, 0)
+		profile := filepath.Join(dirs.SnapSeccompDir, "snap.samba.hook.configure")
+
+		// Verify that profile snap.samba.hook.configure was removed.
+		_, err := os.Stat(profile)
+		c.Check(os.IsNotExist(err), Equals, true)
+		s.RemoveSnap(c, snapInfo)
 	}
 }
 
 func (s *backendSuite) TestRealDefaultTemplateIsNormallyUsed(c *C) {
-	snapInfo, err := snap.InfoFromSnapYaml([]byte(sambaYamlV1))
-	c.Assert(err, IsNil)
+	snapInfo := snaptest.MockInfo(c, backendtest.SambaYamlV1, nil)
 	// NOTE: we don't call seccomp.MockTemplate()
-	err = s.backend.Setup(snapInfo, false, s.repo)
+	err := s.Backend.Setup(snapInfo, interfaces.ConfinementOptions{}, s.Repo)
 	c.Assert(err, IsNil)
 	profile := filepath.Join(dirs.SnapSeccompDir, "snap.samba.smbd")
 	data, err := ioutil.ReadFile(profile)
@@ -162,23 +176,33 @@ func (s *backendSuite) TestRealDefaultTemplateIsNormallyUsed(c *C) {
 }
 
 type combineSnippetsScenario struct {
-	devMode bool
+	opts    interfaces.ConfinementOptions
 	snippet string
 	content string
 }
 
 var combineSnippetsScenarios = []combineSnippetsScenario{{
+	opts:    interfaces.ConfinementOptions{},
 	content: "default\n",
 }, {
+	opts:    interfaces.ConfinementOptions{},
 	snippet: "snippet",
 	content: "default\nsnippet\n",
 }, {
-	devMode: true,
+	opts:    interfaces.ConfinementOptions{DevMode: true},
 	content: "@complain\ndefault\n",
 }, {
-	devMode: true,
+	opts:    interfaces.ConfinementOptions{DevMode: true},
 	snippet: "snippet",
 	content: "@complain\ndefault\nsnippet\n",
+}, {
+	opts:    interfaces.ConfinementOptions{Classic: true},
+	snippet: "snippet",
+	content: "@unrestricted\ndefault\nsnippet\n",
+}, {
+	opts:    interfaces.ConfinementOptions{Classic: true, JailMode: true},
+	snippet: "snippet",
+	content: "default\nsnippet\n",
 }}
 
 func (s *backendSuite) TestCombineSnippets(c *C) {
@@ -186,74 +210,20 @@ func (s *backendSuite) TestCombineSnippets(c *C) {
 	restore := seccomp.MockTemplate([]byte("default\n"))
 	defer restore()
 	for _, scenario := range combineSnippetsScenarios {
-		s.iface.PermanentSlotSnippetCallback = func(slot *interfaces.Slot, securitySystem interfaces.SecuritySystem) ([]byte, error) {
+		s.Iface.PermanentSlotSnippetCallback = func(slot *interfaces.Slot, securitySystem interfaces.SecuritySystem) ([]byte, error) {
 			if scenario.snippet == "" {
 				return nil, nil
 			}
 			return []byte(scenario.snippet), nil
 		}
-		snapInfo := s.installSnap(c, scenario.devMode, sambaYamlV1)
+		snapInfo := s.InstallSnap(c, scenario.opts, backendtest.SambaYamlV1, 0)
 		profile := filepath.Join(dirs.SnapSeccompDir, "snap.samba.smbd")
 		data, err := ioutil.ReadFile(profile)
 		c.Assert(err, IsNil)
 		c.Check(string(data), Equals, scenario.content)
 		stat, err := os.Stat(profile)
+		c.Assert(err, IsNil)
 		c.Check(stat.Mode(), Equals, os.FileMode(0644))
-		s.removeSnap(c, snapInfo)
-	}
-}
-
-// Support code for tests
-
-// installSnap "installs" a snap from YAML.
-func (s *backendSuite) installSnap(c *C, devMode bool, snapYaml string) *snap.Info {
-	snapInfo, err := snap.InfoFromSnapYaml([]byte(snapYaml))
-	c.Assert(err, IsNil)
-	s.addPlugsSlots(c, snapInfo)
-	err = s.backend.Setup(snapInfo, devMode, s.repo)
-	c.Assert(err, IsNil)
-	return snapInfo
-}
-
-// updateSnap "updates" an existing snap from YAML.
-func (s *backendSuite) updateSnap(c *C, oldSnapInfo *snap.Info, devMode bool, snapYaml string) *snap.Info {
-	newSnapInfo, err := snap.InfoFromSnapYaml([]byte(snapYaml))
-	c.Assert(err, IsNil)
-	c.Assert(newSnapInfo.Name(), Equals, oldSnapInfo.Name())
-	s.removePlugsSlots(c, oldSnapInfo)
-	s.addPlugsSlots(c, newSnapInfo)
-	err = s.backend.Setup(newSnapInfo, devMode, s.repo)
-	c.Assert(err, IsNil)
-	return newSnapInfo
-}
-
-// removeSnap "removes" an "installed" snap.
-func (s *backendSuite) removeSnap(c *C, snapInfo *snap.Info) {
-	err := s.backend.Remove(snapInfo.Name())
-	c.Assert(err, IsNil)
-	s.removePlugsSlots(c, snapInfo)
-}
-
-func (s *backendSuite) addPlugsSlots(c *C, snapInfo *snap.Info) {
-	for _, plugInfo := range snapInfo.Plugs {
-		plug := &interfaces.Plug{PlugInfo: plugInfo}
-		err := s.repo.AddPlug(plug)
-		c.Assert(err, IsNil)
-	}
-	for _, slotInfo := range snapInfo.Slots {
-		slot := &interfaces.Slot{SlotInfo: slotInfo}
-		err := s.repo.AddSlot(slot)
-		c.Assert(err, IsNil)
-	}
-}
-
-func (s *backendSuite) removePlugsSlots(c *C, snapInfo *snap.Info) {
-	for _, plug := range s.repo.Plugs(snapInfo.Name()) {
-		err := s.repo.RemovePlug(plug.Snap.Name(), plug.Name)
-		c.Assert(err, IsNil)
-	}
-	for _, slot := range s.repo.Slots(snapInfo.Name()) {
-		err := s.repo.RemoveSlot(slot.Snap.Name(), slot.Name)
-		c.Assert(err, IsNil)
+		s.RemoveSnap(c, snapInfo)
 	}
 }
