@@ -29,40 +29,42 @@ var defaultTemplate = []byte(`
 # Usage: common
 #
 
-# Dangerous syscalls that we don't ever want to allow
+# Dangerous syscalls that we don't ever want to allow.
+# Note: may uncomment once ubuntu-core-launcher understands @deny rules and
+# if/when we conditionally deny these in the future.
 
 # kexec
-deny kexec_load
+#@deny kexec_load
 
 # kernel modules
-deny create_module
-deny init_module
-deny finit_module
-deny delete_module
+#@deny create_module
+#@deny init_module
+#@deny finit_module
+#@deny delete_module
 
 # these have a history of vulnerabilities, are not widely used, and
 # open_by_handle_at has been used to break out of docker containers by brute
 # forcing the handle value: http://stealth.openwall.net/xSports/shocker.c
-deny name_to_handle_at
-deny open_by_handle_at
+#@deny name_to_handle_at
+#@deny open_by_handle_at
 
 # Explicitly deny ptrace since it can be abused to break out of the seccomp
 # sandbox
-deny ptrace
+#@deny ptrace
 
 # Explicitly deny capability mknod so apps can't create devices
-deny mknod
-deny mknodat
+#@deny mknod
+#@deny mknodat
 
 # Explicitly deny (u)mount so apps can't change mounts in their namespace
-deny mount
-deny umount
-deny umount2
+#@deny mount
+#@deny umount
+#@deny umount2
 
 # Explicitly deny kernel keyring access
-deny add_key
-deny keyctl
-deny request_key
+#@deny add_key
+#@deny keyctl
+#@deny request_key
 
 # end dangerous syscalls
 
@@ -218,15 +220,18 @@ fadvise64
 fadvise64_64
 arm_fadvise64_64
 
-mbarrier
 mbind
+membarrier
+memfd_create
 mincore
 mkdir
 mkdirat
 mlock
+mlock2
 mlockall
 mmap
 mmap2
+modify_ldt
 mprotect
 
 # LP: #1448184 - these aren't currently mediated by AppArmor. Deny for now
@@ -259,6 +264,7 @@ open
 
 openat
 pause
+personality
 pipe
 pipe2
 poll
@@ -314,6 +320,11 @@ sched_setscheduler
 
 sched_yield
 
+# Allow configuring seccomp filter. This is ok because the kernel enforces that
+# the new filter is a subset of the current filter (ie, no widening
+# permissions)
+seccomp
+
 select
 _newselect
 pselect
@@ -387,6 +398,11 @@ sigwaitinfo
 socket
 connect
 
+# needed by snapctl
+setsockopt
+getsockname
+getpeername
+
 # Per man page, on Linux this is limited to only AF_UNIX so it is ok to have
 # in the default template
 socketpair
@@ -470,4 +486,9 @@ writev
 pwrite
 pwrite64
 pwritev
+
+# FIXME: remove this after LP: #1446748 is implemented
+# This is an older interface and single entry point that can be used instead
+# of socket(), bind(), connect(), etc individually.
+socketcall
 `)
