@@ -20,20 +20,21 @@
 package snap_test
 
 import (
+	"regexp"
 	"testing"
+	"time"
 
 	. "gopkg.in/check.v1"
 
-	"github.com/ubuntu-core/snappy/snap"
-	"github.com/ubuntu-core/snappy/systemd"
-	"github.com/ubuntu-core/snappy/timeout"
+	"github.com/snapcore/snapd/snap"
+	"github.com/snapcore/snapd/systemd"
+	"github.com/snapcore/snapd/timeout"
 )
 
 // Hook up check.v1 into the "go test" runner
 func Test(t *testing.T) { TestingT(t) }
 
-type InfoSnapYamlTestSuite struct {
-}
+type InfoSnapYamlTestSuite struct{}
 
 var _ = Suite(&InfoSnapYamlTestSuite{})
 
@@ -55,10 +56,21 @@ func (s *InfoSnapYamlTestSuite) TestFail(c *C) {
 	c.Assert(err, ErrorMatches, "(?m)info failed to parse:.*")
 }
 
-type YamlSuite struct{}
+type YamlSuite struct {
+	restore func()
+}
 
 var _ = Suite(&YamlSuite{})
 
+func (s *YamlSuite) SetUpTest(c *C) {
+	hookType := snap.NewHookType(regexp.MustCompile(".*"))
+	s.restore = snap.MockSupportedHookTypes([]*snap.HookType{hookType})
+}
+
+func (s *YamlSuite) TearDownTest(c *C) {
+	s.restore()
+}
+
 func (s *YamlSuite) TestUnmarshalGarbage(c *C) {
 	_, err := snap.InfoFromSnapYaml([]byte(`"`))
 	c.Assert(err, ErrorMatches, ".*: yaml: found unexpected end of stream")
@@ -150,6 +162,35 @@ plugs:
 	})
 }
 
+func (s *YamlSuite) TestUnmarshalStandalonePlugWithIntAndListAndMap(c *C) {
+	// NOTE: yaml content cannot use tabs, indent the section with spaces.
+	info, err := snap.InfoFromSnapYaml([]byte(`
+name: snap
+plugs:
+    iface:
+        interface: complex
+        i: 3
+        l: [1,2,3]
+        m:
+          a: A
+          b: B
+`))
+	c.Assert(err, IsNil)
+	c.Check(info.Name(), Equals, "snap")
+	c.Check(info.Plugs, HasLen, 1)
+	c.Check(info.Slots, HasLen, 0)
+	c.Assert(info.Plugs["iface"], DeepEquals, &snap.PlugInfo{
+		Snap:      info,
+		Name:      "iface",
+		Interface: "complex",
+		Attrs: map[string]interface{}{
+			"i": int64(3),
+			"l": []interface{}{int64(1), int64(2), int64(3)},
+			"m": map[string]interface{}{"a": "A", "b": "B"},
+		},
+	})
+}
+
 func (s *YamlSuite) TestUnmarshalLastPlugDefinitionWins(c *C) {
 	// NOTE: yaml content cannot use tabs, indent the section with spaces.
 	info, err := snap.InfoFromSnapYaml([]byte(`
@@ -170,7 +211,7 @@ plugs:
 		Snap:      info,
 		Name:      "net",
 		Interface: "network-client",
-		Attrs:     map[string]interface{}{"attr": 2},
+		Attrs:     map[string]interface{}{"attr": int64(2)},
 	})
 }
 
@@ -204,6 +245,44 @@ apps:
 	})
 }
 
+func (s *YamlSuite) TestUnmarshalGlobalPlugBoundToOneApp(c *C) {
+	// NOTE: yaml content cannot use tabs, indent the section with spaces.
+	info, err := snap.InfoFromSnapYaml([]byte(`
+name: snap
+plugs:
+    network-client:
+apps:
+    with-plug:
+        plugs: [network-client]
+    without-plug:
+`))
+	c.Assert(err, IsNil)
+	c.Check(info.Name(), Equals, "snap")
+	c.Check(info.Plugs, HasLen, 1)
+	c.Check(info.Slots, HasLen, 0)
+	c.Check(info.Apps, HasLen, 2)
+
+	plug := info.Plugs["network-client"]
+	withPlugApp := info.Apps["with-plug"]
+	withoutPlugApp := info.Apps["without-plug"]
+	c.Assert(plug, DeepEquals, &snap.PlugInfo{
+		Snap:      info,
+		Name:      "network-client",
+		Interface: "network-client",
+		Apps:      map[string]*snap.AppInfo{withPlugApp.Name: withPlugApp},
+	})
+	c.Assert(withPlugApp, DeepEquals, &snap.AppInfo{
+		Snap:  info,
+		Name:  "with-plug",
+		Plugs: map[string]*snap.PlugInfo{plug.Name: plug},
+	})
+	c.Assert(withoutPlugApp, DeepEquals, &snap.AppInfo{
+		Snap:  info,
+		Name:  "without-plug",
+		Plugs: map[string]*snap.PlugInfo{},
+	})
+}
+
 func (s *YamlSuite) TestUnmarshalPlugsExplicitlyDefinedExplicitlyBoundToApps(c *C) {
 	// NOTE: yaml content cannot use tabs, indent the section with spaces.
 	info, err := snap.InfoFromSnapYaml([]byte(`
@@ -360,6 +439,32 @@ plugs:
 	c.Assert(err, ErrorMatches, `plug "serial" uses reserved attribute "\$baud-rate"`)
 }
 
+func (s *YamlSuite) TestUnmarshalInvalidPlugAttribute(c *C) {
+	// NOTE: yaml content cannot use tabs, indent the section with spaces.
+	_, err := snap.InfoFromSnapYaml([]byte(`
+name: snap
+plugs:
+    serial:
+        interface: serial-port
+        foo: null
+`))
+	c.Assert(err, ErrorMatches, `attribute "foo" of plug \"serial\": invalid attribute scalar:.*`)
+}
+
+func (s *YamlSuite) TestUnmarshalInvalidAttributeMapKey(c *C) {
+	// NOTE: yaml content cannot use tabs, indent the section with spaces.
+	_, err := snap.InfoFromSnapYaml([]byte(`
+name: snap
+plugs:
+    serial:
+        interface: serial-port
+        bar:
+          baz:
+          - 1: A
+`))
+	c.Assert(err, ErrorMatches, `attribute "bar" of plug \"serial\": non-string key in attribute map: 1`)
+}
+
 // Tests focusing on slots
 
 func (s *YamlSuite) TestUnmarshalStandaloneImplicitSlot(c *C) {
@@ -438,6 +543,34 @@ slots:
 	})
 }
 
+func (s *YamlSuite) TestUnmarshalStandaloneSlotWithIntAndListAndMap(c *C) {
+	// NOTE: yaml content cannot use tabs, indent the section with spaces.
+	info, err := snap.InfoFromSnapYaml([]byte(`
+name: snap
+slots:
+    iface:
+        interface: complex
+        i: 3
+        l: [1,2]
+        m:
+          a: "A"
+`))
+	c.Assert(err, IsNil)
+	c.Check(info.Name(), Equals, "snap")
+	c.Check(info.Plugs, HasLen, 0)
+	c.Check(info.Slots, HasLen, 1)
+	c.Assert(info.Slots["iface"], DeepEquals, &snap.SlotInfo{
+		Snap:      info,
+		Name:      "iface",
+		Interface: "complex",
+		Attrs: map[string]interface{}{
+			"i": int64(3),
+			"l": []interface{}{int64(1), int64(2)},
+			"m": map[string]interface{}{"a": "A"},
+		},
+	})
+}
+
 func (s *YamlSuite) TestUnmarshalLastSlotDefinitionWins(c *C) {
 	// NOTE: yaml content cannot use tabs, indent the section with spaces.
 	info, err := snap.InfoFromSnapYaml([]byte(`
@@ -458,7 +591,7 @@ slots:
 		Snap:      info,
 		Name:      "net",
 		Interface: "network-client",
-		Attrs:     map[string]interface{}{"attr": 2},
+		Attrs:     map[string]interface{}{"attr": int64(2)},
 	})
 }
 
@@ -648,6 +781,272 @@ slots:
 	c.Assert(err, ErrorMatches, `slot "serial" uses reserved attribute "\$baud-rate"`)
 }
 
+func (s *YamlSuite) TestUnmarshalInvalidSlotAttribute(c *C) {
+	// NOTE: yaml content cannot use tabs, indent the section with spaces.
+	_, err := snap.InfoFromSnapYaml([]byte(`
+name: snap
+slots:
+    serial:
+        interface: serial-port
+        foo: null
+`))
+	c.Assert(err, ErrorMatches, `attribute "foo" of slot \"serial\": invalid attribute scalar:.*`)
+}
+
+func (s *YamlSuite) TestUnmarshalHook(c *C) {
+	// NOTE: yaml content cannot use tabs, indent the section with spaces.
+	info, err := snap.InfoFromSnapYaml([]byte(`
+name: snap
+hooks:
+    test-hook:
+`))
+	c.Assert(err, IsNil)
+	c.Check(info.Name(), Equals, "snap")
+	c.Check(info.Plugs, HasLen, 0)
+	c.Check(info.Slots, HasLen, 0)
+	c.Check(info.Apps, HasLen, 0)
+	c.Check(info.Hooks, HasLen, 1)
+
+	hook, ok := info.Hooks["test-hook"]
+	c.Assert(ok, Equals, true, Commentf("Expected hooks to include 'test-hook'"))
+
+	c.Check(hook, DeepEquals, &snap.HookInfo{
+		Snap:  info,
+		Name:  "test-hook",
+		Plugs: nil,
+	})
+}
+
+func (s *YamlSuite) TestUnmarshalUnsupportedHook(c *C) {
+	s.restore()
+	hookType := snap.NewHookType(regexp.MustCompile("not-test-hook"))
+	s.restore = snap.MockSupportedHookTypes([]*snap.HookType{hookType})
+
+	// NOTE: yaml content cannot use tabs, indent the section with spaces.
+	info, err := snap.InfoFromSnapYaml([]byte(`
+name: snap
+hooks:
+    test-hook:
+`))
+	c.Assert(err, IsNil)
+	c.Check(info.Name(), Equals, "snap")
+	c.Check(info.Plugs, HasLen, 0)
+	c.Check(info.Slots, HasLen, 0)
+	c.Check(info.Apps, HasLen, 0)
+	c.Check(info.Hooks, HasLen, 0, Commentf("Expected no hooks to be loaded"))
+}
+
+func (s *YamlSuite) TestUnmarshalHookFiltersOutUnsupportedHooks(c *C) {
+	s.restore()
+	hookType := snap.NewHookType(regexp.MustCompile("test-.*"))
+	s.restore = snap.MockSupportedHookTypes([]*snap.HookType{hookType})
+
+	// NOTE: yaml content cannot use tabs, indent the section with spaces.
+	info, err := snap.InfoFromSnapYaml([]byte(`
+name: snap
+hooks:
+    test-hook:
+    foo-hook:
+`))
+	c.Assert(err, IsNil)
+	c.Check(info.Name(), Equals, "snap")
+	c.Check(info.Plugs, HasLen, 0)
+	c.Check(info.Slots, HasLen, 0)
+	c.Check(info.Apps, HasLen, 0)
+	c.Check(info.Hooks, HasLen, 1)
+
+	hook, ok := info.Hooks["test-hook"]
+	c.Assert(ok, Equals, true, Commentf("Expected hooks to include 'test-hook'"))
+
+	c.Check(hook, DeepEquals, &snap.HookInfo{
+		Snap:  info,
+		Name:  "test-hook",
+		Plugs: nil,
+	})
+}
+
+func (s *YamlSuite) TestUnmarshalHookWithPlug(c *C) {
+	// NOTE: yaml content cannot use tabs, indent the section with spaces.
+	info, err := snap.InfoFromSnapYaml([]byte(`
+name: snap
+hooks:
+    test-hook:
+        plugs: [test-plug]
+`))
+	c.Assert(err, IsNil)
+	c.Check(info.Name(), Equals, "snap")
+	c.Check(info.Plugs, HasLen, 1)
+	c.Check(info.Slots, HasLen, 0)
+	c.Check(info.Apps, HasLen, 0)
+	c.Check(info.Hooks, HasLen, 1)
+
+	plug, ok := info.Plugs["test-plug"]
+	c.Assert(ok, Equals, true, Commentf("Expected plugs to include 'test-plug'"))
+	hook, ok := info.Hooks["test-hook"]
+	c.Assert(ok, Equals, true, Commentf("Expected hooks to include 'test-hook'"))
+
+	c.Check(plug, DeepEquals, &snap.PlugInfo{
+		Snap:      info,
+		Name:      "test-plug",
+		Interface: "test-plug",
+		Hooks:     map[string]*snap.HookInfo{hook.Name: hook},
+	})
+	c.Check(hook, DeepEquals, &snap.HookInfo{
+		Snap:  info,
+		Name:  "test-hook",
+		Plugs: map[string]*snap.PlugInfo{plug.Name: plug},
+	})
+}
+
+func (s *YamlSuite) TestUnmarshalGlobalPlugsBindToHooks(c *C) {
+	// NOTE: yaml content cannot use tabs, indent the section with spaces.
+	info, err := snap.InfoFromSnapYaml([]byte(`
+name: snap
+plugs:
+    test-plug:
+hooks:
+    test-hook:
+`))
+	c.Assert(err, IsNil)
+	c.Check(info.Name(), Equals, "snap")
+	c.Check(info.Plugs, HasLen, 1)
+	c.Check(info.Slots, HasLen, 0)
+	c.Check(info.Apps, HasLen, 0)
+	c.Check(info.Hooks, HasLen, 1)
+
+	plug, ok := info.Plugs["test-plug"]
+	c.Assert(ok, Equals, true, Commentf("Expected plugs to include 'test-plug'"))
+	hook, ok := info.Hooks["test-hook"]
+	c.Assert(ok, Equals, true, Commentf("Expected hooks to include 'test-hook'"))
+
+	c.Check(plug, DeepEquals, &snap.PlugInfo{
+		Snap:      info,
+		Name:      "test-plug",
+		Interface: "test-plug",
+		Hooks:     map[string]*snap.HookInfo{hook.Name: hook},
+	})
+	c.Check(hook, DeepEquals, &snap.HookInfo{
+		Snap:  info,
+		Name:  "test-hook",
+		Plugs: map[string]*snap.PlugInfo{plug.Name: plug},
+	})
+}
+
+func (s *YamlSuite) TestUnmarshalGlobalPlugBoundToOneHook(c *C) {
+	// NOTE: yaml content cannot use tabs, indent the section with spaces.
+	info, err := snap.InfoFromSnapYaml([]byte(`
+name: snap
+plugs:
+    test-plug:
+hooks:
+    with-plug:
+        plugs: [test-plug]
+    without-plug:
+`))
+	c.Assert(err, IsNil)
+	c.Check(info.Name(), Equals, "snap")
+	c.Check(info.Plugs, HasLen, 1)
+	c.Check(info.Slots, HasLen, 0)
+	c.Check(info.Apps, HasLen, 0)
+	c.Check(info.Hooks, HasLen, 2)
+
+	plug := info.Plugs["test-plug"]
+	withPlugHook := info.Hooks["with-plug"]
+	withoutPlugHook := info.Hooks["without-plug"]
+	c.Assert(plug, DeepEquals, &snap.PlugInfo{
+		Snap:      info,
+		Name:      "test-plug",
+		Interface: "test-plug",
+		Hooks:     map[string]*snap.HookInfo{withPlugHook.Name: withPlugHook},
+	})
+	c.Assert(withPlugHook, DeepEquals, &snap.HookInfo{
+		Snap:  info,
+		Name:  "with-plug",
+		Plugs: map[string]*snap.PlugInfo{plug.Name: plug},
+	})
+	c.Assert(withoutPlugHook, DeepEquals, &snap.HookInfo{
+		Snap:  info,
+		Name:  "without-plug",
+		Plugs: map[string]*snap.PlugInfo{},
+	})
+}
+
+func (s *YamlSuite) TestUnmarshalExplicitGlobalPlugBoundToHook(c *C) {
+	// NOTE: yaml content cannot use tabs, indent the section with spaces.
+	info, err := snap.InfoFromSnapYaml([]byte(`
+name: snap
+plugs:
+    test-plug: test-interface
+hooks:
+    test-hook:
+        plugs: ["test-plug"]
+`))
+	c.Assert(err, IsNil)
+	c.Check(info.Name(), Equals, "snap")
+	c.Check(info.Plugs, HasLen, 1)
+	c.Check(info.Slots, HasLen, 0)
+	c.Check(info.Apps, HasLen, 0)
+	c.Check(info.Hooks, HasLen, 1)
+
+	plug, ok := info.Plugs["test-plug"]
+	c.Assert(ok, Equals, true, Commentf("Expected plugs to include 'test-plug'"))
+	hook, ok := info.Hooks["test-hook"]
+	c.Assert(ok, Equals, true, Commentf("Expected hooks to include 'test-hook'"))
+
+	c.Check(plug, DeepEquals, &snap.PlugInfo{
+		Snap:      info,
+		Name:      "test-plug",
+		Interface: "test-interface",
+		Hooks:     map[string]*snap.HookInfo{hook.Name: hook},
+	})
+	c.Check(hook, DeepEquals, &snap.HookInfo{
+		Snap:  info,
+		Name:  "test-hook",
+		Plugs: map[string]*snap.PlugInfo{plug.Name: plug},
+	})
+}
+
+func (s *YamlSuite) TestUnmarshalGlobalPlugBoundToHookNotApp(c *C) {
+	// NOTE: yaml content cannot use tabs, indent the section with spaces.
+	info, err := snap.InfoFromSnapYaml([]byte(`
+name: snap
+plugs:
+    test-plug:
+hooks:
+    test-hook:
+        plugs: [test-plug]
+apps:
+    test-app:
+`))
+	c.Assert(err, IsNil)
+	c.Check(info.Name(), Equals, "snap")
+	c.Check(info.Plugs, HasLen, 1)
+	c.Check(info.Slots, HasLen, 0)
+	c.Check(info.Apps, HasLen, 1)
+	c.Check(info.Hooks, HasLen, 1)
+
+	plug := info.Plugs["test-plug"]
+	hook := info.Hooks["test-hook"]
+	app := info.Apps["test-app"]
+	c.Assert(plug, DeepEquals, &snap.PlugInfo{
+		Snap:      info,
+		Name:      "test-plug",
+		Interface: "test-plug",
+		Apps:      map[string]*snap.AppInfo{},
+		Hooks:     map[string]*snap.HookInfo{hook.Name: hook},
+	})
+	c.Assert(hook, DeepEquals, &snap.HookInfo{
+		Snap:  info,
+		Name:  "test-hook",
+		Plugs: map[string]*snap.PlugInfo{plug.Name: plug},
+	})
+	c.Assert(app, DeepEquals, &snap.AppInfo{
+		Snap:  info,
+		Name:  "test-app",
+		Plugs: map[string]*snap.PlugInfo{},
+	})
+}
+
 func (s *YamlSuite) TestUnmarshalComplexExample(c *C) {
 	// NOTE: yaml content cannot use tabs, indent the section with spaces.
 	info, err := snap.InfoFromSnapYaml([]byte(`
@@ -655,24 +1054,29 @@ name: foo
 version: 1.2
 summary: foo app
 type: app
+epoch: 1*
+confinement: devmode
 description: |
     Foo provides useful services
 apps:
     daemon:
        command: foo --daemon
        plugs: [network, network-bind]
-       slots: [foo-socket]
+       slots: [foo-socket-slot]
     foo:
        command: fooctl
-       plugs: [foo-socket]
+       plugs: [foo-socket-plug]
+hooks:
+    test-hook:
+       plugs: [foo-socket-plug]
 plugs:
-    foo-socket:
+    foo-socket-plug:
         interface: socket
         # $protocol: foo
     logging:
         interface: syslog
 slots:
-    foo-socket:
+    foo-socket-slot:
         interface: socket
         path: $SNAP_DATA/socket
         protocol: foo
@@ -683,6 +1087,8 @@ slots:
 	c.Check(info.Name(), Equals, "foo")
 	c.Check(info.Version, Equals, "1.2")
 	c.Check(info.Type, Equals, snap.TypeApp)
+	c.Check(info.Epoch, Equals, "1*")
+	c.Check(info.Confinement, Equals, snap.DevModeConfinement)
 	c.Check(info.Summary(), Equals, "foo app")
 	c.Check(info.Description(), Equals, "Foo provides useful services\n")
 	c.Check(info.Apps, HasLen, 2)
@@ -694,11 +1100,12 @@ slots:
 
 	app1 := info.Apps["daemon"]
 	app2 := info.Apps["foo"]
+	hook := info.Hooks["test-hook"]
 	plug1 := info.Plugs["network"]
 	plug2 := info.Plugs["network-bind"]
-	plug3 := info.Plugs["foo-socket"]
+	plug3 := info.Plugs["foo-socket-plug"]
 	plug4 := info.Plugs["logging"]
-	slot1 := info.Slots["foo-socket"]
+	slot1 := info.Slots["foo-socket-slot"]
 	slot2 := info.Slots["tracing"]
 
 	// app1 ("daemon") has three plugs ("network", "network-bind", "logging")
@@ -727,6 +1134,15 @@ slots:
 	c.Check(app2.Slots, DeepEquals, map[string]*snap.SlotInfo{
 		slot2.Name: slot2})
 
+	// hook1 has two plugs ("foo-socket", "logging"). The plug "logging" is
+	// global while "foo-socket" is hook-bound.
+
+	c.Assert(hook, NotNil)
+	c.Check(hook.Snap, Equals, info)
+	c.Check(hook.Name, Equals, "test-hook")
+	c.Check(hook.Plugs, DeepEquals, map[string]*snap.PlugInfo{
+		plug3.Name: plug3, plug4.Name: plug4})
+
 	// plug1 ("network") is implicitly defined and app-bound to "daemon"
 
 	c.Assert(plug1, Not(IsNil))
@@ -751,7 +1167,7 @@ slots:
 
 	c.Assert(plug3, Not(IsNil))
 	c.Check(plug3.Snap, Equals, info)
-	c.Check(plug3.Name, Equals, "foo-socket")
+	c.Check(plug3.Name, Equals, "foo-socket-plug")
 	c.Check(plug3.Interface, Equals, "socket")
 	c.Check(plug3.Attrs, HasLen, 0)
 	c.Check(plug3.Label, Equals, "")
@@ -772,7 +1188,7 @@ slots:
 
 	c.Assert(slot1, Not(IsNil))
 	c.Check(slot1.Snap, Equals, info)
-	c.Check(slot1.Name, Equals, "foo-socket")
+	c.Check(slot1.Name, Equals, "foo-socket-slot")
 	c.Check(slot1.Interface, Equals, "socket")
 	c.Check(slot1.Attrs, DeepEquals, map[string]interface{}{
 		"protocol": "foo", "path": "$SNAP_DATA/socket"})
@@ -802,6 +1218,24 @@ version: 1.0
 	c.Assert(info.Type, Equals, snap.TypeApp)
 }
 
+func (s *YamlSuite) TestSnapYamlEpochDefault(c *C) {
+	y := []byte(`name: binary
+version: 1.0
+`)
+	info, err := snap.InfoFromSnapYaml(y)
+	c.Assert(err, IsNil)
+	c.Assert(info.Epoch, Equals, "0")
+}
+
+func (s *YamlSuite) TestSnapYamlConfinementDefault(c *C) {
+	y := []byte(`name: binary
+version: 1.0
+`)
+	info, err := snap.InfoFromSnapYaml(y)
+	c.Assert(err, IsNil)
+	c.Assert(info.Confinement, Equals, snap.StrictConfinement)
+}
+
 func (s *YamlSuite) TestSnapYamlMultipleArchitecturesParsing(c *C) {
 	y := []byte(`name: binary
 version: 1.0
@@ -891,50 +1325,108 @@ apps:
  svc:
    command: svc1
    description: svc one
-   stop-timeout: 25
+   stop-timeout: 25s
    daemon: forking
    stop-command: stop-cmd
    post-stop-command: post-stop-cmd
    restart-condition: on-abnormal
-   socket-mode: socket_mode
-   listen-stream: listen_stream
    bus-name: busName
-   socket: yes
 `)
 	info, err := snap.InfoFromSnapYaml(y)
 	c.Assert(err, IsNil)
 	c.Check(info.Apps, DeepEquals, map[string]*snap.AppInfo{
 		"svc": {
-			Snap:        info,
-			Name:        "svc",
-			Command:     "svc1",
-			Daemon:      "forking",
-			RestartCond: systemd.RestartOnAbnormal,
-			// XXX: stop-timeout seems broken in term of parsing
-			StopTimeout:  timeout.Timeout(25),
-			Stop:         "stop-cmd",
-			PostStop:     "post-stop-cmd",
-			Socket:       true,
-			SocketMode:   "socket_mode",
-			ListenStream: "listen_stream",
-			BusName:      "busName",
+			Snap:            info,
+			Name:            "svc",
+			Command:         "svc1",
+			Daemon:          "forking",
+			RestartCond:     systemd.RestartOnAbnormal,
+			StopTimeout:     timeout.Timeout(25 * time.Second),
+			StopCommand:     "stop-cmd",
+			PostStopCommand: "post-stop-cmd",
+			BusName:         "busName",
 		},
 	})
 }
 
-// legacy
-
-func (s *YamlSuite) TestLegacyParsing(c *C) {
-	y := []byte(`name: gadget-test
+func (s *YamlSuite) TestSnapYamlGlobalEnvironment(c *C) {
+	y := []byte(`
+name: foo
 version: 1.0
-gadget:
-  store:
-    id: my-store
-type: gadget
+environment:
+ foo: bar
+ baz: boom
+`)
+	info, err := snap.InfoFromSnapYaml(y)
+	c.Assert(err, IsNil)
+	c.Assert(info.Environment, DeepEquals, map[string]string{
+		"foo": "bar",
+		"baz": "boom",
+	})
+}
+
+func (s *YamlSuite) TestSnapYamlPerAppEnvironment(c *C) {
+	y := []byte(`
+name: foo
+version: 1.0
+apps:
+ foo:
+  environment:
+   k1: v1
+   k2: v2
+`)
+	info, err := snap.InfoFromSnapYaml(y)
+	c.Assert(err, IsNil)
+	c.Assert(info.Apps["foo"].Environment, DeepEquals, map[string]string{
+		"k1": "v1",
+		"k2": "v2",
+	})
+}
+
+// classic confinement
+func (s *YamlSuite) TestClassicConfinement(c *C) {
+	y := []byte(`
+name: foo
+confinement: classic
+`)
+	info, err := snap.InfoFromSnapYaml(y)
+	c.Assert(err, IsNil)
+	c.Assert(info.Confinement, Equals, snap.ClassicConfinement)
+}
+
+func (s *YamlSuite) TestSnapYamlAliases(c *C) {
+	y := []byte(`
+name: foo
+version: 1.0
+apps:
+  foo:
+    aliases: [foo]
+  bar:
+    aliases: [bar, bar1]
 `)
 	info, err := snap.InfoFromSnapYaml(y)
 	c.Assert(err, IsNil)
 
-	c.Assert(info.Legacy, NotNil)
-	c.Check(info.Legacy.Gadget.Store.ID, Equals, "my-store")
+	c.Check(info.Apps["foo"].Aliases, DeepEquals, []string{"foo"})
+	c.Check(info.Apps["bar"].Aliases, DeepEquals, []string{"bar", "bar1"})
+
+	c.Check(info.Aliases, DeepEquals, map[string]*snap.AppInfo{
+		"foo":  info.Apps["foo"],
+		"bar":  info.Apps["bar"],
+		"bar1": info.Apps["bar"],
+	})
+}
+
+func (s *YamlSuite) TestSnapYamlAliasesConflict(c *C) {
+	y := []byte(`
+name: foo
+version: 1.0
+apps:
+  foo:
+    aliases: [bar]
+  bar:
+    aliases: [bar]
+`)
+	_, err := snap.InfoFromSnapYaml(y)
+	c.Assert(err, ErrorMatches, `cannot set "bar" as alias for both ("foo" and "bar"|"bar" and "foo")`)
 }
